@@ -1,7 +1,6 @@
+import warnings
 import collections
 from copy import deepcopy
-from bson import ObjectId
-from bson.py3compat import string_type
 
 from .base import (
     BaseObject,
@@ -10,16 +9,20 @@ from .base import (
     validate_ok_for_replace,
     validate_list_or_none,
     validate_boolean,
-    command_coder,
 )
 
 from .cursor import MontyCursor
-from .engine.core import FieldWalker, Weighted
+from .engine.field_walker import FieldWalker
+from .engine.weighted import Weighted
 from .engine.queries import QueryFilter
 from .engine.update import Updator
-from .engine.helpers import is_duckument_type, Counter
-
-from .storage.errors import StorageDuplicateKeyError
+from .types import (
+    ObjectId,
+    string_types,
+    is_duckument_type,
+    Counter,
+)
+from .storage import StorageDuplicateKeyError
 from .errors import (
     DuplicateKeyError,
     BulkWriteError,
@@ -32,6 +35,31 @@ from .results import (BulkWriteResult,
                       UpdateResult)
 
 
+NotImplementeds = {
+    "aggregate",
+    "aggregate_raw_batches",
+    "with_options",
+    "bulk_write",
+    "watch",
+    "find_raw_batches",
+    "find_one_and_delete",
+    "find_one_and_replace",
+    "find_one_and_update",
+    "create_index",
+    "create_indexes",
+    "drop_index",
+    "drop_indexes",
+    "reindex",
+    "list_indexes",
+    "index_information",
+    "rename",
+    "options",
+    "map_reduce",
+    "inline_map_reduce",
+    "parallel_scan",
+}
+
+
 class MontyCollection(BaseObject):
 
     def __init__(self, database, name, create=False,
@@ -61,6 +89,9 @@ class MontyCollection(BaseObject):
         return not self == other
 
     def __getattr__(self, name):
+        if name in NotImplementeds:
+            raise NotImplementedError("'MontyCollection.%s' is NOT "
+                                      "implemented !" % name)
         if name.startswith('_'):
             full_name = ".".join((self._name, name))
             raise AttributeError(
@@ -89,12 +120,6 @@ class MontyCollection(BaseObject):
         """
         return self._database
 
-    def with_options(self, codec_options=None, write_concern=None):
-        raise NotImplementedError("Not implemented.")
-
-    def bulk_write(self, requests, ordered=True):
-        raise NotImplementedError("Not implemented.")
-
     def insert_one(self,
                    document,
                    bypass_document_validation=False, *args, **kwargs):
@@ -172,8 +197,6 @@ class MontyCollection(BaseObject):
         validate_ok_for_replace(replacement)
         validate_boolean("upsert", upsert)
 
-        filter, = command_coder(filter, codec_op=self._database.codec_options)
-
         if bypass_document_validation:
             pass
 
@@ -254,12 +277,6 @@ class MontyCollection(BaseObject):
         validate_list_or_none('array_filters', array_filters)
         validate_boolean('upsert', upsert)
 
-        filter, update = command_coder(filter, update,
-                                       codec_op=self._database.codec_options)
-        if array_filters:
-            array_filters = command_coder(
-                *array_filters, codec_op=self._database.codec_options)
-
         if bypass_document_validation:
             pass
 
@@ -291,12 +308,6 @@ class MontyCollection(BaseObject):
         validate_list_or_none('array_filters', array_filters)
         validate_boolean('upsert', upsert)
 
-        filter, update = command_coder(filter, update,
-                                       codec_op=self._database.codec_options)
-        if array_filters:
-            array_filters = command_coder(
-                *array_filters, codec_op=self._database.codec_options)
-
         if bypass_document_validation:
             pass
 
@@ -355,33 +366,10 @@ class MontyCollection(BaseObject):
 
         return DeleteResult(raw_result)
 
-    def aggregate(self, pipeline, session=None, **kwargs):
-        # return CommandCursor
-        raise NotImplementedError("Not implemented.")
-
-    def aggregate_raw_batches(self, pipeline, **kwargs):
-        # return RawBatchCursor
-        raise NotImplementedError("Not implemented.")
-
-    def watch(self,
-              pipeline=None,
-              full_document='default',
-              resume_after=None,
-              max_await_time_ms=None,
-              batch_size=None,
-              collation=None,
-              session=None):
-        # return a changeStreamCursor
-        raise NotImplementedError("Not implemented.")
-
     def find(self, *args, **kwargs):
         # return a cursor
         return MontyCursor(self, *args, **kwargs)
 
-    def find_raw_batches(self, *args, **kwargs):
-        # return RawBatchCursor
-        raise NotImplementedError("Not implemented.")
-
     def find_one(self, filter=None, *args, **kwargs):
         """
         """
@@ -394,32 +382,34 @@ class MontyCollection(BaseObject):
             return result
         return None
 
-    def find_one_and_delete(self, filter, projection=None, sort=None):
-        raise NotImplementedError("Not implemented.")
-
-    def find_one_and_replace(self,
-                             filter,
-                             replacement,
-                             projection=None,
-                             sort=None, *args, **kwargs):
-        raise NotImplementedError("Not implemented.")
-
-    def find_one_and_update(self,
-                            filter,
-                            update,
-                            projection=None,
-                            sort=None, *args, **kwargs):
-        raise NotImplementedError("Not implemented.")
-
     def count(self, filter=None):
-        raise NotImplementedError("Not implemented.")
+        warnings.warn("count is deprecated. Use Collection.count_documents "
+                      "instead.", DeprecationWarning, stacklevel=2)
+
+        documents = self._storage.query(MontyCursor(self), 0)
+        if filter:
+            return self.__count_with_filter(documents, filter)
+        else:
+            return len(list(documents))
+
+    def count_documents(self, filter):
+        documents = self._storage.query(MontyCursor(self), 0)
+        return self.__count_with_filter(documents, filter)
+
+    def __count_with_filter(self, documents, filter):
+        count = 0
+        queryfilter = QueryFilter(filter)
+        for doc in documents:
+            if queryfilter(doc):
+                count += 1
+        return count
 
     def distinct(self, key, filter=None, **kwargs):
         """
         """
-        if not isinstance(key, string_type):
+        if not isinstance(key, string_types):
             raise TypeError("key must be an "
-                            "instance of %s" % (string_type.__name__,))
+                            "instance of %s" % (string_types.__name__,))
 
         result = list()
 
@@ -427,7 +417,7 @@ class MontyCollection(BaseObject):
             fieldwalker = FieldWalker(doc)
             fieldvalues = fieldwalker.go(key).get().value
             res = list()
-            for v in fieldvalues.values:
+            for v in fieldvalues.iter_flat():
                 weighted = Weighted(v)
                 if weighted not in result:
                     res.append(weighted)
@@ -446,28 +436,6 @@ class MontyCollection(BaseObject):
 
         return [weighted.value for weighted in result]
 
-    def create_index(self, keys):
-        raise NotImplementedError("Not implemented.")
-
-    def create_indexes(self, indexes):
-        raise NotImplementedError("Not implemented.")
-
-    def drop_index(self, index_or_name):
-        raise NotImplementedError("Not implemented.")
-
-    def drop_indexes(self):
-        raise NotImplementedError("Not implemented.")
-
-    def reindex(self):
-        raise NotImplementedError("Not implemented.")
-
-    def list_indexes(self):
-        # return a commandCursor
-        raise NotImplementedError("Not implemented.")
-
-    def index_information(self):
-        raise NotImplementedError("Not implemented.")
-
     def drop(self):
         self._database.drop_collection(self._name)
 
@@ -480,27 +448,3 @@ class MontyCollection(BaseObject):
                              *args, **kwargs)
         else:
             self.insert_one(to_save, *args, **kwargs)
-
-    def rename(self, new_name, session=None, **kwargs):
-        raise NotImplementedError("Not implemented.")
-
-    def options(self, session=None):
-        raise NotImplementedError("Not implemented.")
-
-    def map_reduce(self,
-                   map,
-                   reduce,
-                   out,
-                   full_response=False,
-                   session=None, **kwargs):
-        raise NotImplementedError("Not implemented.")
-
-    def inline_map_reduce(self,
-                          map,
-                          reduce,
-                          full_response=False,
-                          session=None, **kwargs):
-        raise NotImplementedError("Not implemented.")
-
-    def parallel_scan(self, num_cursors, session=None, **kwargs):
-        raise NotImplementedError("Not implemented.")
