@@ -4,37 +4,47 @@ from copy import deepcopy
 from datetime import datetime
 from collections import Mapping
 
-from bson import SON
-from bson.py3compat import integer_types, string_type
-from bson.regex import Regex, str_flags_to_int
-from bson.min_key import MinKey
-from bson.max_key import MaxKey
-from bson.decimal128 import Decimal128
-from bson.timestamp import Timestamp
-from bson.objectid import ObjectId
-from bson.binary import Binary
-from bson.code import Code
-from bson.int64 import Int64
-
 from ..errors import OperationFailure
 
-from .core import (
-    gravity,
-    FieldWalker,
+from .field_walker import FieldWalker
+from .weighted import (
     Weighted,
+    gravity,
     _cmp_decimal,
     _decimal128_INF,
     _decimal128_NaN_ls,
 )
+from ..types import (
+    integer_types,
+    string_types,
+    SON,
+    Regex,
+    MinKey,
+    MaxKey,
+    Decimal128,
+    Timestamp,
+    ObjectId,
+    Binary,
+    Code,
+    Int64,
 
-from .helpers import (
     RE_PATTERN_TYPE,
+
     is_duckument_type,
+    is_integer_type,
     is_pattern_type,
     keep,
     compare_documents,
+    re_str_flags_to_int,
 )
 
+from . import MONTY_MONGO_COMPAT_36
+
+
+def validate_sort_specifier(sort):
+    if not (is_integer_type(sort) and sort in (1, -1)):
+        raise OperationFailure("bad sort specification", code=2)
+
 
 def ordering(fieldwalkers, order, doc_type=None):
     """
@@ -43,13 +53,15 @@ def ordering(fieldwalkers, order, doc_type=None):
     pre_sect_stack = []
 
     for path, revr in order.items():
+        validate_sort_specifier(revr)
+
         is_reverse = bool(1 - revr)
         value_stack = []
 
         for indx, fieldwalker in enumerate(fieldwalkers):
             # get field value
             fieldwalker = FieldWalker(fieldwalker.doc, doc_type).go(path).get()
-            values = fieldwalker.value.values
+            values = list(fieldwalker.value.iter_flat())
             if values:
                 value = tuple([Weighted(val) for val in values])
 
@@ -58,7 +70,7 @@ def ordering(fieldwalkers, order, doc_type=None):
                     # or largest member
                     value = max(value) if is_reverse else min(value)
 
-            elif not fieldwalker.value.exists:
+            elif not fieldwalker.value.is_exists():
                 value = Weighted(None)
 
             else:
@@ -118,12 +130,12 @@ class LogicBox(list):
         self.implicity = implicity
         self._logic = {
 
-            "$and": self.__call_and,
-            "$or": self.__call_or,
-            "$nor": self.__call_nor,
-            "$not": self.__call_not,
+            "$and": self._call_and,
+            "$or": self._call_or,
+            "$nor": self._call_nor,
+            "$not": self._call_not,
 
-            "$elemMatch": self.__call_elemMatch,
+            "$elemMatch": self._call_elemMatch,
 
         }
 
@@ -164,36 +176,35 @@ class LogicBox(list):
         try:
             return self._logic[self.theme](fieldwalker)
         except KeyError:
-            return self.__call_field(fieldwalker)
+            return self._call_field(fieldwalker)
 
-    def __gen(self, fieldwalker):
+    def _gen(self, fieldwalker):
         return (cond(fieldwalker) for cond in self[:])
 
-    def __call_field(self, fieldwalker):
+    def _call_field(self, fieldwalker):
         """Entering document field context before process"""
         with fieldwalker.go(self.theme).get():
-            return all(self.__gen(fieldwalker))
+            return all(self._gen(fieldwalker))
 
-    def __call_elemMatch(self, fieldwalker):
+    def _call_elemMatch(self, fieldwalker):
         """"""
         with fieldwalker.value as field_value:
-            for value in field_value.iter_arrays():
-                for v in value:
-                    field_value._value_iter = lambda: iter([v])
-                    if all(self.__gen(fieldwalker)):
-                        return True
+            for elem in field_value.iter_elements():
+                field_value.change_iter(lambda: iter([elem]))
+                if all(self._gen(fieldwalker)):
+                    return True
 
-    def __call_and(self, fieldwalker):
-        return all(self.__gen(fieldwalker))
+    def _call_and(self, fieldwalker):
+        return all(self._gen(fieldwalker))
 
-    def __call_or(self, fieldwalker):
-        return any(self.__gen(fieldwalker))
+    def _call_or(self, fieldwalker):
+        return any(self._gen(fieldwalker))
 
-    def __call_nor(self, fieldwalker):
-        return not any(self.__gen(fieldwalker))
+    def _call_nor(self, fieldwalker):
+        return not any(self._gen(fieldwalker))
 
-    def __call_not(self, fieldwalker):
-        return not all(self.__gen(fieldwalker))
+    def _call_not(self, fieldwalker):
+        return not all(self._gen(fieldwalker))
 
 
 class QueryFilter(object):
@@ -448,7 +459,8 @@ def _modify_regex_optins(sub_spec):
         "flags": flags
     }
 
-    if "#" in new_sub_spec["$regex"]["pattern"].rsplit("\n")[-1]:
+    if (MONTY_MONGO_COMPAT_36
+            and "#" in new_sub_spec["$regex"]["pattern"].rsplit("\n")[-1]):
         # (NOTE) davidlatwe:
         #   if pound(#) char exists in $regex string value and not ends with
         #   newline(\n), Mongo raise error. (but the message seems incomplete)
@@ -486,7 +498,7 @@ def _eq_match(fieldwalker, query):
 
     else:
         if query is None:
-            return fieldwalker.value.null_or_missing
+            return fieldwalker.value.null_or_missing()
 
         if isinstance(query, Decimal128):
             query = _cmp_decimal(query)
@@ -607,7 +619,7 @@ def _in_match(fieldwalker, query):
 
     for q in q_regex:
         for value in fieldwalker.value:
-            if isinstance(value, string_type) and q.search(value):
+            if isinstance(value, string_types) and q.search(value):
                 return True
 
 
@@ -726,7 +738,7 @@ Field-level Query Operators
 def parse_exists(query):
     @keep(query)
     def _exists(fieldwalker):
-        return fieldwalker.value.exists == bool(query)
+        return fieldwalker.value.is_exists() == bool(query)
 
     return _exists
 
@@ -793,9 +805,9 @@ def obj_to_bson_type_id(obj):
     try:
         type_id = BSON_TYPE_ID[type(obj)]
     except KeyError:
-        if isinstance(obj, Code):  # also an instance of string_type
+        if isinstance(obj, Code):  # also an instance of string_types
             type_id = 13 if obj.scope is None else 15
-        elif isinstance(obj, string_type):
+        elif isinstance(obj, string_types):
             type_id = 2
         elif isinstance(obj, bytes):
             type_id = 5
@@ -820,7 +832,7 @@ def parse_type(query):
 
         int_types = []
         for q in query:
-            if isinstance(q, string_type):
+            if isinstance(q, string_types):
                 try:
                     int_types.append(BSON_TYPE_ALIAS_ID[q])
                 except KeyError:
@@ -842,7 +854,7 @@ def parse_type(query):
 
     @keep(query)
     def _type(fieldwalker):
-        if fieldwalker.value.exists:
+        if fieldwalker.value.is_exists():
             bids = get_bson_type_id_set(fieldwalker.value)
             return bids.intersection(query)
 
@@ -859,22 +871,22 @@ def parse_regex(query):
     if isinstance(query, Regex):
         q = query.try_compile()
     else:
-        if not isinstance(query["pattern"], string_type):
+        if not isinstance(query["pattern"], string_types):
             raise OperationFailure("$regex has to be a string")
-        if not isinstance(query["flags"], (string_type, _FALG)):
+        if not isinstance(query["flags"], (string_types, _FALG)):
             raise OperationFailure("$options has to be a string")
 
         if isinstance(query["flags"], _FALG):
             flags = query["flags"].retrieve
         else:
-            flags = str_flags_to_int(query["flags"])
+            flags = re_str_flags_to_int(query["flags"])
 
         q = re.compile(query["pattern"], flags)
 
     @keep(query)
     def _regex(fieldwalker):
         for value in fieldwalker.value:
-            if isinstance(value, (string_type, bytes)) and q.search(value):
+            if isinstance(value, (string_types, bytes)) and q.search(value):
                 return True
 
     return _regex
